@@ -10,51 +10,49 @@ from django.db import models
 from django.db.models import Q
 
 
-
 class PageManager(PublisherManager):
     """Use draft() and public() methods for accessing the corresponding
     instances.
     """
-    
+
     def get_query_set(self):
         """Change standard model queryset to our own.
         """
         return PageQuerySet(self.model)
-    
+
     def drafts(self):
         return super(PageManager, self).drafts().exclude(
             publisher_state=self.model.PUBLISHER_STATE_DELETE
         )
-    
+
     def public(self):
         return super(PageManager, self).public().exclude(
             publisher_state=self.model.PUBLISHER_STATE_DELETE
         )
 
-    
-    # !IMPORTANT: following methods always return access to draft instances, 
+    # !IMPORTANT: following methods always return access to draft instances,
     # take care on what you do one them. use Page.objects.public() for accessing
     # the published page versions
-    
-    # Just some of the queryset methods are implemented here, access queryset 
+
+    # Just some of the queryset methods are implemented here, access queryset
     # for more getting more supporting methods.
-    
+
     # TODO: check which from following methods are really required to be on
     # manager, maybe some of them can be just accessible over queryset...?
-    
-    def on_site(self, site=None): 
+
+    def on_site(self, site=None):
         return self.get_query_set().on_site(site)
-        
+
     def root(self):
         """
         Return a queryset with pages that don't have parents, a.k.a. root. For
         current site - used in frontend
         """
         return self.get_query_set().root()
-    
+
     def all_root(self):
         """
-        Return a queryset with pages that don't have parents, a.k.a. root. For 
+        Return a queryset with pages that don't have parents, a.k.a. root. For
         all sites - used in frontend
         """
         return self.get_query_set().all_root()
@@ -67,11 +65,11 @@ class PageManager(PublisherManager):
 
     def published(self, site=None):
         return self.get_query_set().published(site)
-        
+
     def expired(self):
         return self.drafts().expired()
-        
-#    - seems this is not used anymore...  
+
+#    - seems this is not used anymore...
 #    def get_pages_with_application(self, path, language):
 #        """Returns all pages containing application for current path, or
 #        any parrent. Returned list is sorted by path length, longer path first.
@@ -85,53 +83,51 @@ class PageManager(PublisherManager):
 #        # add proper ordering
 #        app_pages.query.order_by.extend(('LENGTH(`cms_title`.`path`) DESC',))
 #        return app_pages
-    
 
     def get_all_pages_with_application(self):
         """Returns all pages containing applications for all sites.
-        
-        Doesn't cares about the application language. 
+
+        Doesn't cares about the application language.
         """
         return self.get_query_set().filter(title_set__application_urls__gt='').distinct()
-    
+
     def get_home(self, site=None):
         return self.get_query_set().get_home(site)
 
-    
     def search(self, q, language=None, current_site_only=True):
         """Simple search function
-        
+
         Plugins can define a 'search_fields' tuple similar to ModelAdmin classes
         """
         from cms.plugin_pool import plugin_pool
         qs = self.get_query_set()
         if settings.CMS_MODERATOR:
             qs = qs.public()
-        
+
         if current_site_only:
             site = Site.objects.get_current()
             qs = qs.filter(site=site)
-        
+
         qt = Q(title_set__title__icontains=q)
-        
+
         # find 'searchable' plugins and build query
         qp = Q()
         plugins = plugin_pool.get_all_plugins()
         for plugin in plugins:
-            c = plugin.model
-            if hasattr(c, 'search_fields'):
-                for field in c.search_fields:
+            cmsplugin = plugin.model
+            if hasattr(cmsplugin, 'search_fields'):
+                for field in cmsplugin.search_fields:
                     qp |= Q(**{'placeholders__cmsplugin__%s__%s__icontains' % \
-                                   (c.__name__.lower(), field):q})
+                                   (cmsplugin.__name__.lower(), field):q})
         if language:
             qt &= Q(title_set__language=language)
             qp &= Q(cmsplugin__language=language)
-         
+
         qs = qs.filter(qt | qp)
-            
+
         return qs.distinct()
-        
-        
+
+
 class TitleManager(PublisherManager):
     def get_title(self, page, language, language_fallback=False):
         """
@@ -146,20 +142,20 @@ class TitleManager(PublisherManager):
                 try:
                     titles = self.filter(page=page)
                     fallbacks = get_fallback_languages(language)
-                    for l in fallbacks:
+                    for lang in fallbacks:
                         for title in titles:
-                            if l == title.language:
+                            if lang == title.language:
                                 return title
                     return None
                 except self.model.DoesNotExist:
                     pass
             else:
                 raise
-        return None        
-    
+        return None
+
     def get_page_slug(self, slug, site=None):
         """
-        Returns the latest slug for the given slug and checks if it's available 
+        Returns the latest slug for the given slug and checks if it's available
         on the current site.
         """
         if not site:
@@ -168,19 +164,19 @@ class TitleManager(PublisherManager):
             titles = self.filter(
                 slug=slug,
                 page__site=site,
-            ).select_related()#'page')
+            ).select_related()  # 'page')
         except self.model.DoesNotExist:
             return None
         else:
             return titles
-            
+
     # created new public method to meet test case requirement and to get a list of titles for published pages
     def public(self):
         return self.get_query_set().filter(page__publisher_is_draft=False, page__published=True)
-        
+
     def drafts(self):
         return self.get_query_set().filter(page__publisher_is_draft=True)
-        
+
     def set_or_create(self, request, page, form, language):
         """
         set or create a title for a particular page and language
@@ -217,7 +213,7 @@ class TitleManager(PublisherManager):
                     if value:
                         data[field] = value
             return self.create(**data)
-        
+
         for name in base_fields:
             value = cleaned_data.get(name, None)
             setattr(obj, name, value)
@@ -234,57 +230,59 @@ class TitleManager(PublisherManager):
 # Permissions
 ################################################################################
 
+
 class BasicPagePermissionManager(models.Manager):
     """Global page permission manager accessible under objects.
-    
-    !IMPORTANT: take care, PagePermissionManager extends this manager 
+
+    !IMPORTANT: take care, PagePermissionManager extends this manager
     """
     def with_user(self, user):
         """Get all objects for given user, also takes look if user is in some
         group.
         """
         return self.filter(Q(user=user) | Q(group__user=user))
-    
+
     def with_can_change_permissions(self, user):
         """Set of objects on which user haves can_change_permissions. !But only
-        the ones on which is this assigned directly. For getting reall 
-        permissions use page.permissions manager. 
+        the ones on which is this assigned directly. For getting reall
+        permissions use page.permissions manager.
         """
         return self.with_user(user).filter(can_change_permissions=True)
-    
+
+
 class PagePermissionManager(BasicPagePermissionManager):
     """Page permission manager accessible under objects.
     """
     def subordinate_to_user(self, user):
-        """Get all page permission objects on which user/group is lover in 
+        """Get all page permission objects on which user/group is lover in
         hierarchy then given user and given user can change permissions on them.
-        
+
         !IMPORTANT, but exclude objects with given user, or any group containing
-        this user - he can't be able to change his own permissions, because if 
-        he does, and removes some permissions from himself, he will not be able 
-        to add them anymore. 
-        
+        this user - he can't be able to change his own permissions, because if
+        he does, and removes some permissions from himself, he will not be able
+        to add them anymore.
+
         Example:
                                        A
                                     /    \
                                   user    B,E
                                 /     \
                               C,X     D,Y
-            
-            Gives permission nodes C,X,D,Y under user, so he can edit 
+
+            Gives permission nodes C,X,D,Y under user, so he can edit
             permissions if he haves can_change_permission.
-                  
+
         Example:
                                       A,Y
                                     /    \
                                   user    B,E,X
                                 /     \
                               C,X     D,Y
-                              
+
             Gives permission nodes C,D under user, so he can edit, but not
             anymore to X,Y, because this users are on the same level or higher
             in page hierarchy. (but only if user have can_change_permission)
-        
+
         Example:
                                         A
                                     /      \
@@ -293,18 +291,18 @@ class PagePermissionManager(BasicPagePermissionManager):
                               C,X     D,Y    user
                                             /    \
                                            I      J,A
-            
-            User permissions can be assigned to multiple page nodes, so merge of 
-            all of them is required. In this case user can see permissions for 
+
+            User permissions can be assigned to multiple page nodes, so merge of
+            all of them is required. In this case user can see permissions for
             users C,X,D,Y,I,J but not A, because A user in higher in hierarchy.
-        
-        If permission object holds group, this permission object can be visible 
-        to user only if all of the group members are lover in hierarchy. If any 
+
+        If permission object holds group, this permission object can be visible
+        to user only if all of the group members are lover in hierarchy. If any
         of members is higher then given user, this entry must stay invisible.
-        
+
         If user is superuser, or haves global can_change_permission permissions,
         show him everything.
-        
+
         Result of this is used in admin for page permissions inline.
         """
         from cms.models import GlobalPagePermission, Page
@@ -312,7 +310,7 @@ class PagePermissionManager(BasicPagePermissionManager):
             GlobalPagePermission.objects.with_can_change_permissions(user):
             # everything for those guys
                 return self.all()
-        
+
         # get user level
         from cms.utils.permissions import get_user_permission_level
         try:
@@ -322,33 +320,33 @@ class PagePermissionManager(BasicPagePermissionManager):
         # get current site
         site = Site.objects.get_current()
         # get all permissions
-        page_id_allow_list = Page.permissions.get_change_permissions_id_list(user,site)
-        
-        # get permission set, but without objects targeting user, or any group 
+        page_id_allow_list = Page.permissions.get_change_permissions_id_list(user, site)
+
+        # get permission set, but without objects targeting user, or any group
         # in which he can be
         qs = self.filter(
-            page__id__in=page_id_allow_list, 
+            page__id__in=page_id_allow_list,
             page__level__gte=user_level,
         )
         qs = qs.exclude(user=user).exclude(group__user=user)
         return qs
-    
+
     def for_page(self, page):
-        """Returns queryset containing all instances somehow connected to given 
+        """Returns queryset containing all instances somehow connected to given
         page. This includes permissions to page itself and permissions inherited
         from higher pages.
-        
+
         NOTE: this returns just PagePermission instances, to get complete access
         list merge return of this function with Global permissions.
         """
         from cms.models import ACCESS_DESCENDANTS, ACCESS_CHILDREN,\
-            ACCESS_PAGE_AND_CHILDREN, ACCESS_PAGE_AND_DESCENDANTS 
+            ACCESS_PAGE_AND_CHILDREN, ACCESS_PAGE_AND_DESCENDANTS
         # code taken from
         # https://github.com/divio/django-cms/issues/1113#issuecomment-3376790
         q_tree = Q(page__tree_id=page.tree_id)
         q_page = Q(page=page)
-        
-        # NOTE:  '... or 0' is used for test cases, 
+
+        # NOTE:  '... or 0' is used for test cases,
         #        if the page is not saved through mptt
         left_right = {
               'page__%s__lte' % page._mptt_meta.left_attr: getattr(page, page._mptt_meta.left_attr) or 0,
@@ -357,73 +355,71 @@ class PagePermissionManager(BasicPagePermissionManager):
         q_parents = Q(**left_right)
         q_desc = (Q(page__level__lt=page.level) & (Q(grant_on=ACCESS_DESCENDANTS) | Q(grant_on=ACCESS_PAGE_AND_DESCENDANTS)))
         q_kids = (Q(page__level=page.level - 1) & (Q(grant_on=ACCESS_CHILDREN) | Q(grant_on=ACCESS_PAGE_AND_CHILDREN)))
-        q = q_tree & q_parents & (q_page | q_desc | q_kids)
-        return self.filter(q).order_by('page__level')
-        
+        query = q_tree & q_parents & (q_page | q_desc | q_kids)
+        return self.filter(query).order_by('page__level')
+
 
 class PagePermissionsPermissionManager(models.Manager):
     """Page permissions permission manager.
-    
+
     !IMPORTANT: this actually points to Page model, not to PagePermission.
     Seems this will be better approach. Accessible under permissions.
-    
+
     Maybe this even shouldn't be a manager - it mixes different models together.
     """
     # we will return this in case we have a superuser, or permissions are not
     # enabled/configured in settings
     GRANT_ALL = 'All'
-    
+
     def get_publish_id_list(self, user, site):
         """
         Give a list of page where the user has publish rights or the string "All" if
         the user has all rights.
         """
         return self.__get_id_list(user, site, "can_publish")
-    
-    
+
     def get_change_id_list(self, user, site):
         """
         Give a list of page where the user has edit rights or the string "All" if
         the user has all rights.
         """
         return self.__get_id_list(user, site, "can_change")
-    
-    
+
     def get_add_id_list(self, user, site):
         """
-        Give a list of page where the user has add page rights or the string 
+        Give a list of page where the user has add page rights or the string
         "All" if the user has all rights.
         """
         return self.__get_id_list(user, site, "can_add")
-    
+
     def get_delete_id_list(self, user, site):
         """
         Give a list of page where the user has delete rights or the string "All" if
         the user has all rights.
         """
         return self.__get_id_list(user, site, "can_delete")
-    
+
     def get_advanced_settings_id_list(self, user, site):
         """
-        Give a list of page where the user can change advanced settings or the 
+        Give a list of page where the user can change advanced settings or the
         string "All" if the user has all rights.
         """
         return self.__get_id_list(user, site, "can_change_advanced_settings")
-    
+
     def get_change_permissions_id_list(self, user, site):
         """Give a list of page where the user can change permissions.
         """
         return self.__get_id_list(user, site, "can_change_permissions")
-    
+
     def get_move_page_id_list(self, user, site):
         """Give a list of pages which user can move.
         """
         return self.__get_id_list(user, site, "can_move_page")
-    
+
     def get_moderate_id_list(self, user, site):
-        """Give a list of pages which user can moderate. If moderation isn't 
-        installed, nobody can moderate. 
-        """        
+        """Give a list of pages which user can moderate. If moderation isn't
+        installed, nobody can moderate.
+        """
         if not settings.CMS_MODERATOR:
             return []
         return self.__get_id_list(user, site, "can_moderate")
@@ -432,13 +428,13 @@ class PagePermissionsPermissionManager(models.Manager):
         """Give a list of pages which user can view.
         """
         return self.__get_id_list(user, site, "can_view")
-    
+
     '''
     def get_change_list_id_list(self, user, site):
         """This is used just in admin now. Gives all ids where user haves can_edit
         and can_add merged together.
-        
-        There is for sure a better way how to do this over sql, need to be 
+
+        There is for sure a better way how to do this over sql, need to be
         optimized...
         """
         can_change = self.get_change_id_list(user)
@@ -447,14 +443,14 @@ class PagePermissionsPermissionManager(models.Manager):
             # GRANT_ALL case
             page_id_list = can_change
         else:
-            permission_set = filter(lambda i: not i is PagePermissionsPermissionManager.GRANT_ALL, [can_change, can_add])   
+            permission_set = filter(lambda i: not i is PagePermissionsPermissionManager.GRANT_ALL, [can_change, can_add])
             if len(permission_set) is 1:
                 page_id_list = permission_set[0]
             else:
                 page_id_list = list(set(can_change).union(set(can_add)))
         return page_id_list
-    '''    
-    
+    '''
+
     def __get_id_list(self, user, site, attr):
         from cms.models import (GlobalPagePermission, PagePermission,
                                 MASK_PAGE, MASK_CHILDREN, MASK_DESCENDANTS)
@@ -462,7 +458,7 @@ class PagePermissionsPermissionManager(models.Manager):
             if not user.is_authenticated() or not user.is_staff:
                 return []
         if user.is_superuser or not settings.CMS_PERMISSION:
-            # got superuser, or permissions aren't enabled? just return grant 
+            # got superuser, or permissions aren't enabled? just return grant
             # all mark
             return PagePermissionsPermissionManager.GRANT_ALL
         # read from cache if posssible
@@ -472,7 +468,7 @@ class PagePermissionsPermissionManager(models.Manager):
         # check global permissions
         global_permissions = GlobalPagePermission.objects.with_user(user)
         if global_permissions.filter(**{
-                attr: True, 'sites__in':[site]
+                attr: True, 'sites__in': [site]
             }).exists():
             # user or his group are allowed to do `attr` action
             # !IMPORTANT: page permissions must not override global permissions
@@ -483,7 +479,7 @@ class PagePermissionsPermissionManager(models.Manager):
         qs.order_by('page__tree_id', 'page__level', 'page__lft')
         # default is denny...
         page_id_allow_list = []
-        
+
         for permission in qs:
             if getattr(permission, attr):
 
