@@ -4,8 +4,7 @@ from cms.plugin_processors import (plugin_meta_context_processor,
     mark_safe_plugin_processor)
 from cms.utils import get_language_from_request
 from cms.utils.django_load import iterload_objects
-from cms.utils.placeholder import (get_page_from_placeholder_if_exists, 
-    get_placeholder_conf)
+from cms.utils.placeholder import get_placeholder_conf
 from django.conf import settings
 from django.template import Template, Context
 from django.template.defaultfilters import title
@@ -72,15 +71,15 @@ def render_plugins(plugins, context, placeholder, processors=None):
     This is the main plugin rendering utility function, use this function rather than
     Plugin.render_plugin().
     """
-    c = []
+    out = []
     total = len(plugins)
     for index, plugin in enumerate(plugins):
         plugin._render_meta.total = total 
         plugin._render_meta.index = index
         context.push()
-        c.append(plugin.render_plugin(context, placeholder, processors=processors))
+        out.append(plugin.render_plugin(context, placeholder, processors=processors))
         context.pop()
-    return c
+    return out
 
 def render_placeholder(placeholder, context_to_copy, name_fallback="Placeholder"):
     """
@@ -92,7 +91,7 @@ def render_placeholder(placeholder, context_to_copy, name_fallback="Placeholder"
     context.push()
     request = context['request']
     plugins = [plugin for plugin in get_plugins(request, placeholder)]
-    page = get_page_from_placeholder_if_exists(placeholder)
+    page = placeholder.page if placeholder else None
     if page:
         template = page.template
     else:
@@ -109,7 +108,7 @@ def render_placeholder(placeholder, context_to_copy, name_fallback="Placeholder"
         if not key in context:
             context[key] = value
 
-    c = []
+    content = []
 
     # Prepend frontedit toolbar output if applicable
     edit = False
@@ -124,8 +123,8 @@ def render_placeholder(placeholder, context_to_copy, name_fallback="Placeholder"
     else:
         processors = None 
 
-    c.extend(render_plugins(plugins, context, placeholder, processors))
-    content = "".join(c)
+    content.extend(render_plugins(plugins, context, placeholder, processors))
+    content = "".join(content)
     if edit:
         content = render_placeholder_toolbar(placeholder, context, content, name_fallback)
     context.pop()
@@ -134,7 +133,7 @@ def render_placeholder(placeholder, context_to_copy, name_fallback="Placeholder"
 def render_placeholder_toolbar(placeholder, context, content, name_fallback=None):
     from cms.plugin_pool import plugin_pool
     request = context['request']
-    page = get_page_from_placeholder_if_exists(placeholder)
+    page = placeholder.page if placeholder else None
     if not page:
         page = getattr(request, 'current_page', None)
     if page:
@@ -142,6 +141,7 @@ def render_placeholder_toolbar(placeholder, context, content, name_fallback=None
         if name_fallback and not placeholder:
             placeholder = Placeholder.objects.create(slot=name_fallback)
             page.placeholders.add(placeholder)
+            placeholder.page = page
     else:
         template = None
     if placeholder:
