@@ -19,8 +19,8 @@ from cms.test_utils.project.pluginapp.models import Article, Section
 from cms.test_utils.project.pluginapp.plugins.manytomany_rel.models import (
     ArticlePluginModel)
 from cms.test_utils.testcases import (CMSTestCase, URL_CMS_PAGE, 
-    URL_CMS_PAGE_ADD, URL_CMS_PLUGIN_ADD, URL_CMS_PLUGIN_EDIT, URL_CMS_PAGE_CHANGE, 
-    URL_CMS_PLUGIN_REMOVE)
+    URL_CMS_PAGE_ADD, URL_CMS_PLUGIN_ADD, URL_CMS_PLUGIN_EDIT, URL_CMS_PAGE_CHANGE, URL_CMS_PLUGIN_REMOVE)
+from cms.sitemaps.cms_sitemap import CMSSitemap
 from cms.test_utils.util.context_managers import SettingsOverride
 from cms.utils.copy_plugins import copy_plugins_to
 from django.conf import settings
@@ -31,6 +31,7 @@ from django.core.management import call_command
 from django.forms.widgets import Media
 from django.test.testcases import TestCase
 import os
+import datetime
 
 
 class DumbFixturePlugin(CMSPluginBase):
@@ -76,6 +77,31 @@ class PluginsTestBaseCase(CMSTestCase):
 
 
 class PluginsTestCase(PluginsTestBaseCase):
+
+    def _create_text_plugin_on_page(self, page):
+        plugin_data = {
+            'plugin_type':"TextPlugin",
+            'language':settings.LANGUAGES[0][0],
+            'placeholder':page.placeholders.get(slot="body").pk,
+        }
+        response = self.client.post(URL_CMS_PLUGIN_ADD, plugin_data)
+        self.assertEquals(response.status_code, 200)
+        created_plugin_id = int(response.content)
+        self.assertEquals(created_plugin_id, CMSPlugin.objects.all()[0].pk)
+        return created_plugin_id
+
+    def _edit_text_plugin(self, plugin_id, text):
+        edit_url = "%s%s/" % (URL_CMS_PLUGIN_EDIT, plugin_id) 
+        response = self.client.get(edit_url)
+        self.assertEquals(response.status_code, 200)
+        data = {
+            "body": text
+        }
+        response = self.client.post(edit_url, data)
+        self.assertEquals(response.status_code, 200)
+        txt = Text.objects.get(pk=plugin_id)
+        return txt
+
     def test_add_edit_plugin(self):
         """
         Test that you can add a text plugin
@@ -84,35 +110,45 @@ class PluginsTestCase(PluginsTestBaseCase):
         page_data = self.get_new_page_data()
         response = self.client.post(URL_CMS_PAGE_ADD, page_data)
         page = Page.objects.all()[0]
-        plugin_data = {
-            'plugin_type':"TextPlugin",
-            'language':settings.LANGUAGES[0][0],
-            'placeholder':page.placeholders.get(slot="body").pk,
-        }
-        response = self.client.post(URL_CMS_PLUGIN_ADD, plugin_data)
-        self.assertEquals(response.status_code, 200)
-        self.assertEquals(int(response.content), CMSPlugin.objects.all()[0].pk)
+        created_plugin_id = self._create_text_plugin_on_page(page)
         # now edit the plugin
-        edit_url = URL_CMS_PLUGIN_EDIT + response.content + "/"
-        response = self.client.get(edit_url)
-        self.assertEquals(response.status_code, 200)
-        data = {
-            "body":"Hello World"
-        }
-        response = self.client.post(edit_url, data)
-        self.assertEquals(response.status_code, 200)
-        txt = Text.objects.all()[0]
+        txt = self._edit_text_plugin(created_plugin_id, "Hello World")
         self.assertEquals("Hello World", txt.body)
         # edit body, but click cancel button
         data = {
             "body":"Hello World!!",
             "_cancel":True,
         }
-        response = self.client.post(edit_url, data)
+        response = self.client.post(URL_CMS_PAGE_ADD, data)
         self.assertEquals(response.status_code, 200)
         txt = Text.objects.all()[0]
         self.assertEquals("Hello World", txt.body)
 
+
+    def test_plugin_order(self):
+        """
+        Test that plugin position is saved after creation
+        """
+        page_en = create_page("PluginOrderPage", "col_two.html", "en",
+                              slug="page1",published=True, in_navigation=True)
+        ph_en = page_en.placeholders.get(slot="col_left")
+
+        # We check created objects and objects from the DB to be sure the position value
+        # has been saved correctly
+        text_plugin_1 = add_plugin(ph_en, "TextPlugin", "en", body="I'm the first")
+        text_plugin_2 = add_plugin(ph_en, "TextPlugin", "en", body="I'm the second")
+        db_plugin_1 = CMSPlugin.objects.get(pk=text_plugin_1.pk)
+        db_plugin_2 = CMSPlugin.objects.get(pk=text_plugin_2.pk)
+
+        with SettingsOverride(CMS_MODERATOR=False, CMS_PERMISSION=False):
+            self.assertEqual(text_plugin_1.position,1)
+            self.assertEqual(db_plugin_1.position,1)
+            self.assertEqual(text_plugin_2.position,2)
+            self.assertEqual(db_plugin_2.position,2)
+            ## Finally we render the placeholder to test the actual content
+            rendered_placeholder = ph_en.render(self.get_context(page_en.get_absolute_url()),None)
+            self.assertEquals(rendered_placeholder,"I'm the firstI'm the second")
+
     def test_add_cancel_plugin(self):
         """
         Test that you can cancel a new plugin before editing and 
@@ -142,6 +178,62 @@ class PluginsTestCase(PluginsTestBaseCase):
         self.assertEquals(response.status_code, 200)
         self.assertEquals(0, Text.objects.count())
 
+    def test_add_text_plugin_empty_tag(self):
+        """
+        Test that you can add a text plugin
+        """
+        # add a new text plugin
+        page_data = self.get_new_page_data()
+        response = self.client.post(URL_CMS_PAGE_ADD, page_data)
+        page = Page.objects.all()[0]
+        plugin_data = {
+            'plugin_type':"TextPlugin",
+            'language':settings.LANGUAGES[0][0],
+            'placeholder':page.placeholders.get(slot="body").pk,
+            }
+        response = self.client.post(URL_CMS_PLUGIN_ADD, plugin_data)
+        self.assertEquals(response.status_code, 200)
+        self.assertEquals(int(response.content), CMSPlugin.objects.all()[0].pk)
+        # now edit the plugin
+        edit_url = URL_CMS_PLUGIN_EDIT + response.content + "/"
+        response = self.client.get(edit_url)
+        self.assertEquals(response.status_code, 200)
+        data = {
+            "body":'<div class="someclass"></div><p>foo</p>'
+        }
+        response = self.client.post(edit_url, data)
+        self.assertEquals(response.status_code, 200)
+        txt = Text.objects.all()[0]
+        self.assertEquals('<div class="someclass"></div><p>foo</p>', txt.body)
+
+    def test_add_text_plugin_html_sanitizer(self):
+        """
+        Test that you can add a text plugin
+        """
+        # add a new text plugin
+        page_data = self.get_new_page_data()
+        response = self.client.post(URL_CMS_PAGE_ADD, page_data)
+        page = Page.objects.all()[0]
+        plugin_data = {
+            'plugin_type':"TextPlugin",
+            'language':settings.LANGUAGES[0][0],
+            'placeholder':page.placeholders.get(slot="body").pk,
+            }
+        response = self.client.post(URL_CMS_PLUGIN_ADD, plugin_data)
+        self.assertEquals(response.status_code, 200)
+        self.assertEquals(int(response.content), CMSPlugin.objects.all()[0].pk)
+        # now edit the plugin
+        edit_url = URL_CMS_PLUGIN_EDIT + response.content + "/"
+        response = self.client.get(edit_url)
+        self.assertEquals(response.status_code, 200)
+        data = {
+            "body":'<script>var bar="hacked"</script>'
+        }
+        response = self.client.post(edit_url, data)
+        self.assertEquals(response.status_code, 200)
+        txt = Text.objects.all()[0]
+        self.assertEquals('&lt;script&gt;var bar="hacked"&lt;/script&gt;', txt.body)
+
     def test_copy_plugins(self):
         """
         Test that copying plugins works as expected.
@@ -375,6 +467,39 @@ class PluginsTestCase(PluginsTestBaseCase):
             response = self.client.get(page.get_absolute_url())
             self.assertTrue('%scms/js/libs/jquery.tweet.js' % settings.STATIC_URL in response.content, response.content)
 
+    def test_render_textplugin(self):
+        # Setup
+        page = create_page("render test", "nav_playground.html", "en")
+        ph = page.placeholders.get(slot="body")
+        text_plugin = add_plugin(ph, "TextPlugin", "en", body="Hello World")
+        link_plugins = []
+        for i in range(0, 10):
+            link_plugins.append(add_plugin(ph, "LinkPlugin", "en",
+                target=text_plugin,
+                name="A Link %d" % i,
+                url="http://django-cms.org"))
+            text_plugin.text.body += '<img src="/static/cms/images/plugins/link.png" alt="Link - %s" id="plugin_obj_%d" title="Link - %s" />' % (
+                link_plugins[-1].name,
+                link_plugins[-1].pk,
+                link_plugins[-1].name,
+            )
+        text_plugin.save()
+        txt = text_plugin.text
+        ph = Placeholder.objects.get(pk=ph.pk)
+        with self.assertNumQueries(2):
+            # 1 query for the CMSPlugin objects,
+            # 1 query for each type of child object (1 in this case, all are Link plugins)
+            txt.body = plugin_tags_to_admin_html(
+                '\n'.join(["{{ plugin_object %d }}" % l.cmsplugin_ptr_id
+                           for l in link_plugins]))
+        txt.save()
+        text_plugin = self.reload(text_plugin)
+
+        with self.assertNumQueries(2):
+            rendered = text_plugin.render_plugin(placeholder=ph)
+        for i in range(0, 10):
+            self.assertTrue('A Link %d' % i in rendered)
+
     def test_copy_textplugin(self):
         """
         Test that copying of textplugins replaces references to copied plugins
@@ -450,9 +575,39 @@ class PluginsTestCase(PluginsTestBaseCase):
 
         new_plugin = Text.objects.get(pk=6)
         idlist = sorted(plugin_tags_to_id_list(new_plugin.body))
-        expected = sorted([u'4', u'5'])
+        expected = sorted([4, 5])
         self.assertEquals(idlist, expected)
 
+    def test_empty_plugin_is_ignored(self):
+        page = create_page("page", "nav_playground.html", "en")
+
+        placeholder = page.placeholders.get(slot='body')
+
+        plugin = CMSPlugin(
+            plugin_type='TextPlugin',
+            placeholder=placeholder,
+            position=1,
+            language=self.FIRST_LANG)
+        plugin.insert_at(None, position='last-child', save=True)
+
+        # this should not raise any errors, but just ignore the empty plugin
+        out = placeholder.render(self.get_context(), width=300)
+        self.assertFalse(len(out))
+        self.assertFalse(len(placeholder._en_plugins_cache))
+
+    def test_editing_plugin_changes_page_modification_time_in_sitemap(self):
+        now = datetime.datetime.now()
+        one_day_ago = now - datetime.timedelta(days=1)
+        page = create_page("page", "nav_playground.html", "en", published=True, publication_date=now)
+        page.creation_date = one_day_ago
+        page.changed_date = one_day_ago
+
+        plugin_id = self._create_text_plugin_on_page(page)
+        plugin = self._edit_text_plugin(plugin_id, "fnord")
+
+        actual_last_modification_time = CMSSitemap().lastmod(page)
+        self.assertEqual(plugin.changed_date, actual_last_modification_time)
+
 
 class FileSystemPluginTests(PluginsTestBaseCase):
     def setUp(self):
@@ -486,6 +641,8 @@ class FileSystemPluginTests(PluginsTestBaseCase):
         self.assertNotEquals(plugin.get_icon_url().find('jpg'), -1)
 
 
+
+
 class PluginManyToManyTestCase(PluginsTestBaseCase):
 
     def setUp(self):
@@ -647,7 +804,7 @@ class PluginManyToManyTestCase(PluginsTestBaseCase):
         db_counts = [plugin.sections.count() for plugin in ArticlePluginModel.objects.all()]
         expected = [self.section_count for i in range(len(db_counts))]
         self.assertEqual(expected, db_counts)
-        
+
 class PluginsMetaOptionsTests(TestCase):
     ''' TestCase set for ensuring that bugs like #992 are caught '''
 
@@ -726,8 +883,39 @@ class LinkPluginTestCase(PluginsTestBaseCase):
     def test_does_not_verify_existance_of_url(self):
         form = LinkForm(
             {'name': 'Linkname', 'url': 'http://www.nonexistant.test'})
-        self.assertEquals(form.is_valid(), True)
+        self.assertTrue(form.is_valid())
 
+    def test_opens_in_same_window_by_default(self):
+        """Could not figure out how to render this plugin
+
+        Checking only for the values in the model"""
+        form = LinkForm({'name': 'Linkname',
+            'url': 'http://www.nonexistant.test'})
+        link = form.save()
+        self.assertEquals(link.target, '')
+
+    def test_open_in_blank_window(self):
+        form = LinkForm({'name': 'Linkname',
+            'url': 'http://www.nonexistant.test', 'target' : '_blank'})
+        link = form.save()
+        self.assertEquals(link.target, '_blank')
+
+    def test_open_in_parent_window(self):
+        form = LinkForm({'name': 'Linkname',
+            'url': 'http://www.nonexistant.test', 'target' : '_parent'})
+        link = form.save()
+        self.assertEquals(link.target, '_parent')
+
+    def test_open_in_top_window(self):
+        form = LinkForm({'name': 'Linkname',
+            'url': 'http://www.nonexistant.test', 'target' : '_top'})
+        link = form.save()
+        self.assertEquals(link.target, '_top')
+
+    def test_open_in_nothing_else(self):
+        form = LinkForm({'name': 'Linkname',
+            'url': 'http://www.nonexistant.test', 'target' : 'artificial'})
+        self.assertFalse(form.is_valid())
 
 class NoDatabasePluginTests(TestCase):
     def test_render_meta_is_unique(self):
